@@ -15,8 +15,12 @@
 
 #![no_std]
 #![cfg_attr(docsrs, feature(doc_cfg))]
-#![forbid(unsafe_code)]
-#![doc(html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo_small.png")]
+#![forbid(unsafe_code, clippy::unwrap_used)]
+#![doc(
+    html_logo_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
+    html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/media/8f1a9894/logo.svg",
+    html_root_url = "https://docs.rs/aead/0.4.1"
+)]
 #![warn(missing_docs, rust_2018_idioms)]
 
 #[cfg(feature = "alloc")]
@@ -29,11 +33,18 @@ extern crate std;
 #[cfg_attr(docsrs, doc(cfg(feature = "dev")))]
 pub mod dev;
 
+#[cfg(feature = "stream")]
+#[cfg_attr(docsrs, doc(cfg(feature = "stream")))]
+pub mod stream;
+
 pub use generic_array::{self, typenum::consts};
 
 #[cfg(feature = "heapless")]
 pub use heapless;
 
+#[cfg(feature = "rand_core")]
+use rand_core::{CryptoRng, RngCore};
+
 use core::fmt;
 use generic_array::{typenum::Unsigned, ArrayLength, GenericArray};
 
@@ -61,10 +72,10 @@ impl std::error::Error for Error {}
 pub type Key<A> = GenericArray<u8, <A as NewAead>::KeySize>;
 
 /// Nonce: single-use value for ensuring ciphertexts are unique
-pub type Nonce<NonceSize> = GenericArray<u8, NonceSize>;
+pub type Nonce<A> = GenericArray<u8, <A as AeadCore>::NonceSize>;
 
 /// Tag: authentication code which ensures ciphertexts are authentic
-pub type Tag<TagSize> = GenericArray<u8, TagSize>;
+pub type Tag<A> = GenericArray<u8, <A as AeadCore>::TagSize>;
 
 /// Instantiate either a stateless [`Aead`] or stateful [`AeadMut`] algorithm.
 pub trait NewAead {
@@ -74,10 +85,10 @@ pub trait NewAead {
     /// Create a new AEAD instance with the given key.
     fn new(key: &Key<Self>) -> Self;
 
-    /// Create new AEAD instance from key with variable size.
+    /// Create new AEAD instance from key given as a byte slice..
     ///
     /// Default implementation will accept only keys with length equal to `KeySize`.
-    fn new_varkey(key: &[u8]) -> Result<Self, Error>
+    fn new_from_slice(key: &[u8]) -> Result<Self, Error>
     where
         Self: Sized,
     {
@@ -87,15 +98,22 @@ pub trait NewAead {
             Ok(Self::new(GenericArray::from_slice(key)))
         }
     }
+
+    /// Generate a random key for this AEAD using the provided [`CryptoRng`].
+    #[cfg(feature = "rand_core")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "rand_core")))]
+    fn generate_key(mut rng: impl CryptoRng + RngCore) -> Key<Self> {
+        let mut key = Key::<Self>::default();
+        rng.fill_bytes(&mut key);
+        key
+    }
 }
 
-/// Authenticated Encryption with Associated Data (AEAD) algorithm.
+/// Authenticated Encryption with Associated Data (AEAD) algorithm core trait.
 ///
-/// This trait is intended for use with stateless AEAD algorithms. The
-/// [`AeadMut`] trait provides a stateful interface.
-#[cfg(feature = "alloc")]
-#[cfg_attr(docsrs, doc(cfg(feature = "alloc")))]
-pub trait Aead {
+/// Defines nonce, tag, and overhead sizes that are consumed by various other
+/// `Aead*` traits.
+pub trait AeadCore {
     /// The length of a nonce.
     type NonceSize: ArrayLength<u8>;
 
@@ -105,7 +123,15 @@ pub trait Aead {
     /// The upper bound amount of additional space required to support a
     /// ciphertext vs. a plaintext.
     type CiphertextOverhead: ArrayLength<u8> + Unsigned;
+}
 
+/// Authenticated Encryption with Associated Data (AEAD) algorithm.
+///
+/// This trait is intended for use with stateless AEAD algorithms. The
+/// [`AeadMut`] trait provides a stateful interface.
+#[cfg(feature = "alloc")]
+#[cfg_attr(docsrs, doc(cfg(feature = "alloc")))]
+pub trait Aead: AeadCore {
     /// Encrypt the given plaintext payload, and return the resulting
     /// ciphertext as a vector of bytes.
     ///
@@ -131,7 +157,7 @@ pub trait Aead {
     /// ciphertext message.
     fn encrypt<'msg, 'aad>(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         plaintext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error>;
 
@@ -154,7 +180,7 @@ pub trait Aead {
     /// ciphertext message.
     fn decrypt<'msg, 'aad>(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         ciphertext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error>;
 }
@@ -162,17 +188,7 @@ pub trait Aead {
 /// Stateful Authenticated Encryption with Associated Data algorithm.
 #[cfg(feature = "alloc")]
 #[cfg_attr(docsrs, doc(cfg(feature = "alloc")))]
-pub trait AeadMut {
-    /// The length of a nonce.
-    type NonceSize: ArrayLength<u8>;
-
-    /// The maximum length of the nonce.
-    type TagSize: ArrayLength<u8>;
-
-    /// The upper bound amount of additional space required to support a
-    /// ciphertext vs. a plaintext.
-    type CiphertextOverhead: ArrayLength<u8> + Unsigned;
-
+pub trait AeadMut: AeadCore {
     /// Encrypt the given plaintext slice, and return the resulting ciphertext
     /// as a vector of bytes.
     ///
@@ -180,7 +196,7 @@ pub trait AeadMut {
     /// Associated Additional Data (AAD).
     fn encrypt<'msg, 'aad>(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         plaintext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error>;
 
@@ -191,7 +207,7 @@ pub trait AeadMut {
     /// message payloads and Associated Additional Data (AAD).
     fn decrypt<'msg, 'aad>(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         ciphertext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error>;
 }
@@ -209,7 +225,7 @@ macro_rules! impl_decrypt_in_place {
 
         let tag_pos = $buffer.len() - Self::TagSize::to_usize();
         let (msg, tag) = $buffer.as_mut().split_at_mut(tag_pos);
-        $aead.decrypt_in_place_detached($nonce, $aad, msg, Tag::from_slice(tag))?;
+        $aead.decrypt_in_place_detached($nonce, $aad, msg, Tag::<Self>::from_slice(tag))?;
         $buffer.truncate(tag_pos);
         Ok(())
     }};
@@ -218,17 +234,7 @@ macro_rules! impl_decrypt_in_place {
 /// In-place stateless AEAD trait.
 ///
 /// This trait is both object safe and has no dependencies on `alloc` or `std`.
-pub trait AeadInPlace {
-    /// The length of a nonce.
-    type NonceSize: ArrayLength<u8>;
-
-    /// The maximum length of the nonce.
-    type TagSize: ArrayLength<u8>;
-
-    /// The upper bound amount of additional space required to support a
-    /// ciphertext vs. a plaintext.
-    type CiphertextOverhead: ArrayLength<u8> + Unsigned;
-
+pub trait AeadInPlace: AeadCore {
     /// Encrypt the given buffer containing a plaintext message in-place.
     ///
     /// The buffer must have sufficient capacity to store the ciphertext
@@ -240,7 +246,7 @@ pub trait AeadInPlace {
     /// resulting ciphertext message.
     fn encrypt_in_place(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut dyn Buffer,
     ) -> Result<(), Error> {
@@ -252,10 +258,10 @@ pub trait AeadInPlace {
     /// Encrypt the data in-place, returning the authentication tag
     fn encrypt_in_place_detached(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-    ) -> Result<Tag<Self::TagSize>, Error>;
+    ) -> Result<Tag<Self>, Error>;
 
     /// Decrypt the message in-place, returning an error in the event the
     /// provided authentication tag does not match the given ciphertext.
@@ -264,7 +270,7 @@ pub trait AeadInPlace {
     /// message upon success.
     fn decrypt_in_place(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut dyn Buffer,
     ) -> Result<(), Error> {
@@ -276,27 +282,17 @@ pub trait AeadInPlace {
     /// is modified/unauthentic)
     fn decrypt_in_place_detached(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-        tag: &Tag<Self::TagSize>,
+        tag: &Tag<Self>,
     ) -> Result<(), Error>;
 }
 
 /// In-place stateful AEAD trait.
 ///
 /// This trait is both object safe and has no dependencies on `alloc` or `std`.
-pub trait AeadMutInPlace {
-    /// The length of a nonce.
-    type NonceSize: ArrayLength<u8>;
-
-    /// The maximum length of the nonce.
-    type TagSize: ArrayLength<u8>;
-
-    /// The upper bound amount of additional space required to support a
-    /// ciphertext vs. a plaintext.
-    type CiphertextOverhead: ArrayLength<u8> + Unsigned;
-
+pub trait AeadMutInPlace: AeadCore {
     /// Encrypt the given buffer containing a plaintext message in-place.
     ///
     /// The buffer must have sufficient capacity to store the ciphertext
@@ -308,7 +304,7 @@ pub trait AeadMutInPlace {
     /// resulting ciphertext message.
     fn encrypt_in_place(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut impl Buffer,
     ) -> Result<(), Error> {
@@ -320,10 +316,10 @@ pub trait AeadMutInPlace {
     /// Encrypt the data in-place, returning the authentication tag
     fn encrypt_in_place_detached(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-    ) -> Result<Tag<Self::TagSize>, Error>;
+    ) -> Result<Tag<Self>, Error>;
 
     /// Decrypt the message in-place, returning an error in the event the
     /// provided authentication tag does not match the given ciphertext.
@@ -332,7 +328,7 @@ pub trait AeadMutInPlace {
     /// message upon success.
     fn decrypt_in_place(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut impl Buffer,
     ) -> Result<(), Error> {
@@ -344,22 +340,18 @@ pub trait AeadMutInPlace {
     /// is modified/unauthentic)
     fn decrypt_in_place_detached(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-        tag: &Tag<Self::TagSize>,
+        tag: &Tag<Self>,
     ) -> Result<(), Error>;
 }
 
 #[cfg(feature = "alloc")]
 impl<Alg: AeadInPlace> Aead for Alg {
-    type NonceSize = Alg::NonceSize;
-    type TagSize = Alg::TagSize;
-    type CiphertextOverhead = Alg::CiphertextOverhead;
-
     fn encrypt<'msg, 'aad>(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         plaintext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error> {
         let payload = plaintext.into();
@@ -371,7 +363,7 @@ impl<Alg: AeadInPlace> Aead for Alg {
 
     fn decrypt<'msg, 'aad>(
         &self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         ciphertext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error> {
         let payload = ciphertext.into();
@@ -383,13 +375,9 @@ impl<Alg: AeadInPlace> Aead for Alg {
 
 #[cfg(feature = "alloc")]
 impl<Alg: AeadMutInPlace> AeadMut for Alg {
-    type NonceSize = Alg::NonceSize;
-    type TagSize = Alg::TagSize;
-    type CiphertextOverhead = Alg::CiphertextOverhead;
-
     fn encrypt<'msg, 'aad>(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         plaintext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error> {
         let payload = plaintext.into();
@@ -401,7 +389,7 @@ impl<Alg: AeadMutInPlace> AeadMut for Alg {
 
     fn decrypt<'msg, 'aad>(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         ciphertext: impl Into<Payload<'msg, 'aad>>,
     ) -> Result<Vec<u8>, Error> {
         let payload = ciphertext.into();
@@ -412,13 +400,9 @@ impl<Alg: AeadMutInPlace> AeadMut for Alg {
 }
 
 impl<Alg: AeadInPlace> AeadMutInPlace for Alg {
-    type NonceSize = Alg::NonceSize;
-    type TagSize = Alg::TagSize;
-    type CiphertextOverhead = Alg::CiphertextOverhead;
-
     fn encrypt_in_place(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut impl Buffer,
     ) -> Result<(), Error> {
@@ -427,16 +411,16 @@ impl<Alg: AeadInPlace> AeadMutInPlace for Alg {
 
     fn encrypt_in_place_detached(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-    ) -> Result<Tag<Self::TagSize>, Error> {
+    ) -> Result<Tag<Self>, Error> {
         <Self as AeadInPlace>::encrypt_in_place_detached(self, nonce, associated_data, buffer)
     }
 
     fn decrypt_in_place(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut impl Buffer,
     ) -> Result<(), Error> {
@@ -445,10 +429,10 @@ impl<Alg: AeadInPlace> AeadMutInPlace for Alg {
 
     fn decrypt_in_place_detached(
         &mut self,
-        nonce: &Nonce<Self::NonceSize>,
+        nonce: &Nonce<Self>,
         associated_data: &[u8],
         buffer: &mut [u8],
-        tag: &Tag<Self::TagSize>,
+        tag: &Tag<Self>,
     ) -> Result<(), Error> {
         <Self as AeadInPlace>::decrypt_in_place_detached(self, nonce, associated_data, buffer, tag)
     }
@@ -515,10 +499,7 @@ impl Buffer for Vec<u8> {
 }
 
 #[cfg(feature = "heapless")]
-impl<N> Buffer for heapless::Vec<u8, N>
-where
-    N: heapless::ArrayLength<u8>,
-{
+impl<const N: usize> Buffer for heapless::Vec<u8, N> {
     fn extend_from_slice(&mut self, other: &[u8]) -> Result<(), Error> {
         heapless::Vec::extend_from_slice(self, other).map_err(|_| Error)
     }
